'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _os = require('os');

var _os2 = _interopRequireDefault(_os);

var _crypto = require('crypto');

var _crypto2 = _interopRequireDefault(_crypto);

var _redis = require('redis');

var _redis2 = _interopRequireDefault(_redis);

var _kafkaNode = require('kafka-node');

var _kafkaNode2 = _interopRequireDefault(_kafkaNode);

var _express = require('express');

var _express2 = _interopRequireDefault(_express);

var _v = require('uuid/v4');

var _v2 = _interopRequireDefault(_v);

var _worker = require('./worker.js');

var _worker2 = _interopRequireDefault(_worker);

var _fork = require('./fork.js');

var _fork2 = _interopRequireDefault(_fork);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Coppel = function () {
  function Coppel(config) {
    _classCallCheck(this, Coppel);

    this._hostname = _os2.default.hostname();
    config.KAFKA = config.KAFKA.split('2181').join('9092');

    this._config = config;
    this._config.MS_REDIS = process.env.MS_REDIS || null;
    this._config.TOPIC = config.APPLICATION_APP + '_' + config.APPLICATION_VERSION + '_' + config.APPLICATION_NAME;
    if (config.APPLICATION_APP === 'appcoppel' && config.APPLICATION_VERSION === 'v1') {
      this._config.TOPIC = config.APPLICATION_NAME;
    }

    this._config.DEBUG = this._config.DEBUG && this._config.DEBUG.toString() === 'true';
    this._config.APPLICATION_PUBLIC = this._config.APPLICATION_PUBLIC && this._config.APPLICATION_PUBLIC.toString() === 'true';
    this._config.CLEAR_QUEUE = config.CLEAR_QUEUE !== undefined ? config.CLEAR_QUEUE : true;

    if (process.platform === 'win32') {
      this._config.CLEAR_QUEUE = false;
    }

    if (this._config.APPLICATION_TYPE === 'worker') {
      this._app = new _worker2.default(this._config);
    } else {
      this._app = new _fork2.default(this._config);
    }

    this._app.sendError = this._sendError;

    this._errors = {
      '-99': 'Ocurrio una excepcion, favor de intentar de nuevo.',
      '0': 'Smoke Test',
      '1': 'Este servicio no puede ser consumido de forma externa'
    };

    if (this._config.APPLICATION_TYPE === 'worker') {
      this._app.setErrors(this._errors);
    }

    if (this._config.MS_REDIS) {
      this.redisClient = _redis2.default.createClient({ host: this._config.MS_REDIS });
      this.redisClient.on('ready', function () {
        console.log('Redis UniqueID Connected');
      });
    }
  }

  _createClass(Coppel, [{
    key: 'setErrors',
    value: function setErrors(errors) {
      var newErrros = Object.assign({}, this._errors);
      Object.keys(errors).forEach(function (key) {
        if (!newErrros[key]) {
          newErrros[key] = errors[key];
        }
      });

      this._app.setErrors(newErrros);
    }
  }, {
    key: 'setListener',
    value: function setListener(listener) {
      this._listener = true;
      this._app.setListener(listener);
    }
  }, {
    key: 'setBifurcacion',
    value: function setBifurcacion(fork) {
      if (this._config.APPLICATION_TYPE !== 'worker') {
        this._fork = true;
        this._app.setBifurcacion(fork);
      }
    }
  }, {
    key: '_sendError',
    value: function _sendError(service, transactionId, error) {
      var message = Buffer.from(JSON.stringify({
        id: transactionId,
        servicio: service,
        error: error
      })).toString('base64');

      var producer = this._producer;
      if (!producer) {
        producer = this.producer;
      }

      producer.send([{
        topic: 'Errores_criticos',
        messages: [message],
        key: transactionId
      }], function (err, _) {
        if (err) console.error(err);
      });
    }
  }, {
    key: '_response',
    value: function _response(data, errorCode) {
      var errorResponse = {
        meta: {
          status: errorCode.toString() === '0' ? 'SUCCESS' : 'ERROR',
          id_transaction: data.metadata.id_transaction
        },
        data: {
          response: {
            errorCode: errorCode,
            userMessage: this._errors[errorCode.toString()]
          }
        }
      };
      if (errorCode.toString() === '0') {
        errorResponse.data.response.hostname = this._hostname;
        errorResponse.data.response.code = errorCode;
        delete errorResponse.data.response.errorCode;
      }

      data.response = errorResponse;

      data.metadata.time = new Date().toISOString();
      data.metadata.worker = data.metadata.owner;
      data.metadata.owner = this._config.TOPIC;
      data.metadata.mtype = 'output';

      var uowner = '';
      var uworker = '';
      if ('uowner' in data.metadata) {
        uowner = data.metadata.uowner;
      }

      if ('uworker' in data.metadata) {
        uworker = data.metadata.uworker;
      }

      data.metadata.uworker = uowner;
      data.metadata.uowner = uworker;

      var topic = 'respuesta_' + data.metadata.owner;

      data = {
        _id: data.metadata.id_transaction,
        response: data.response,
        metadata: data.metadata
      };

      if (this._config.DEBUG) {
        console.log('SALIDA [' + topic + ']: ' + JSON.stringify(data));
      }

      data = Buffer.from(JSON.stringify(data)).toString('base64');

      this._producer.send([{
        topic: topic,
        messages: [data]
      }], function (err, _) {
        if (err) console.error(err);
      });
    }
  }, {
    key: 'startHttpHealthCheck',
    value: function startHttpHealthCheck() {
      var _this = this;

      this._appHealthCheck = (0, _express2.default)();
      this._appHealthCheck.get('/', function (req, res) {
        if (_this._producer) {

          var operationID = new Date().getTime().toString();
          var data = {
            data: { smoketest: true },
            headers: { Asynchronous: false, Authorizacion: '' },
            metadata: {
              intents: 0,
              callback: _this._config.TOPIC,
              owner: _this._config.TOPIC,
              uowner: _this._config.TOPIC,
              uworker: _this._config.TOPIC + '_' + operationID,
              worker: _this._config.TOPIC,
              asynchronous: false,
              mtype: 'input',
              bifurcacion: false,
              from: _this._hostname,

              id_operacion: operationID,
              id_transaction: (0, _v2.default)(),
              time: new Date().toISOString()
            }
          };

          var topic = _this._config.TOPIC;
          data = Buffer.from(JSON.stringify(data)).toString('base64');
          _this._producer.send([{
            topic: topic,
            messages: [data]
          }], function (err, _) {
            if (err) {
              console.error("HealthCheck] Error: " + err);
              console.error(err);
              res.status(500).json({ error: err.toString() });
            } else {
              res.json({});
            }
          });
        } else {
          console.error("[HealthCheck] Error: Producer no initialized");
          res.status(500).json({ error: 'Producer not initialized' });
        }
      });

      this._appHealthCheck.listen(80, function () {
        console.log('[HealthCheck] listening on 0.0.0.0:80');
      });
    }
  }, {
    key: 'validateId',
    value: function validateId(base64) {
      var _this2 = this;

      var md5 = _crypto2.default.createHash('md5').update(base64).digest("hex");
      var sha = _crypto2.default.createHash('sha256').update(base64).digest('hex');

      var uniqueID = md5 + '_' + sha;

      return new Promise(function (resolve, reject) {
        _this2.redisClient.get(uniqueID, function (err, reply) {
          if (err) {
            reject(err);
          }
          if (reply) {
            resolve(true);
          } else {
            _this2.redisClient.set(uniqueID, '1', 'EX', 60 * 5);
            resolve(false);
          }
        });
      });
    }
  }, {
    key: 'start',
    value: function start() {
      var _this3 = this;

      if (!this._listener) {
        console.error('Listener not configured');
        return;
      } else if (this._producer) {
        console.error('Microservice already started');
        return;
      } else if (this._config.APPLICATION_TYPE !== 'worker' && !this._fork) {
        console.error('Bifurcacion not configured');
        return;
      }

      this._client = new _kafkaNode2.default.KafkaClient({ kafkaHost: this._config.KAFKA });
      this._producer = new _kafkaNode2.default.HighLevelProducer(this._client);
      this.startHttpHealthCheck();

      this._producer.on('ready', function () {
        var options = {
          kafkaHost: _this3._config.KAFKA,
          groupId: _this3._config.TOPIC
        };

        if (_this3._consumer) {
          return;
        }

        _this3._app.producer = _this3._producer;

        var topics = [_this3._config.TOPIC];
        _this3._app.producer.client.loadMetadataForTopics([], function (error, results) {
          if (!error && results.length > 0) {
            var metadata = results.find(function (el) {
              return el.metadata;
            });
            if (metadata) {
              metadata = metadata.metadata;
              topics = Object.keys(metadata);
            }
          }
        });
        setInterval(function () {
          _this3._app.producer.client.refreshMetadata(topics, function (error) {
            if (error) console.log(error);
          });
        }, 30000);

        _this3._consumer = new _kafkaNode2.default.ConsumerGroup(options, [_this3._config.TOPIC]);
        _this3._consumer.on('connect', function () {
          console.log('Conectado a kafka [' + _this3._config.KAFKA + ' - ' + _this3._config.TOPIC);
        });
        _this3._consumer.on('message', function (message) {
          var base64 = message.value.toString();
          var run = function run(base64) {
            try {
              var data = JSON.parse(Buffer.from(base64, 'base64').toString('utf8'));

              if (data.data.smoketest) {
                return _this3._response(data, 0);
              }

              if (_this3._config.DEBUG && _this3._config.APPLICATION_TYPE === 'worker') {
                console.log('ENTRADA [' + _this3._config.TOPIC + ']: HEADERS: [' + JSON.stringify(data.headers) + '] DATA: [' + JSON.stringify(data.data) + ']');
              }

              if (data.metadata.callback === _this3._config.TOPIC && !_this3._config.APPLICATION_PUBLIC) {
                return _this3._response(data, 1);
              } else if (data.metadata.time.indexOf('Z') > -1 && _this3._config.CLEAR_QUEUE) {
                var date = new Date(data.metadata.time);
                var currDate = new Date();
                var diff = Math.round((currDate.getTime() - date.getTime()) / 60000);
                if (diff >= 5) {
                  return;
                }
              }

              _this3._app.message(data);
            } catch (err) {
              console.error(err);
              console.error('ENTRADA [' + _this3._config.TOPIC + ']: [' + message.value.toString() + ']');
              var transactionId = new Date().getTime();
              _this3._sendError(_this3._config.TOPIC, transactionId, err);
              _this3._response({
                metadata: {
                  owner: _this3._config.TOPIC,
                  id_transaction: 'error_' + transactionId
                },
                headers: {},
                data: {}
              }, -99);
            }
          };

          if (_this3._config.MS_REDIS) {
            _this3.validateId(base64).then(function (exists) {
              if (!exists) {
                run(base64);
              }
            }).catch(function (err) {
              console.error(err);
              console.error('ENTRADA [' + _this3._config.TOPIC + ']: [' + message.value.toString() + ']');
              var transactionId = new Date().getTime();
              _this3._sendError(_this3._config.TOPIC, transactionId, err);
              _this3._response({
                metadata: {
                  owner: _this3._config.TOPIC,
                  id_transaction: 'error_' + transactionId
                },
                headers: {},
                data: {}
              }, -99);
            });
          } else {
            run(base64);
          }
        });
        _this3._consumer.on('error', function (err) {
          console.error(err);
        });
      });

      this._producer.on('error', function (err) {
        console.error(err);
      });
    }
  }]);

  return Coppel;
}();

exports.default = Coppel;