'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Worker = function () {
  function Worker(config) {
    _classCallCheck(this, Worker);

    this.config = config;
  }

  _createClass(Worker, [{
    key: 'getError',
    value: function getError(transactionId, code) {
      var userMessage = '';
      if (!this.errors[code.toString()] || code.toString() === '-99') {
        userMessage = 'Ocurrio una excepcion, favor de intentar de nuevo.';
      } else {
        userMessage = this.errors[code.toString()];
      }

      var response = {
        meta: {
          status: 'ERROR',
          id_transaction: transactionId
        },
        data: {
          response: {
            errorCode: code,
            userMessage: userMessage
          }
        }
      };
      return response;
    }
  }, {
    key: 'setErrors',
    value: function setErrors(errors) {
      this.errors = errors;
    }
  }, {
    key: 'setListener',
    value: function setListener(listener) {
      this.listener = listener;
    }
  }, {
    key: 'send',
    value: function send(data, message) {
      var response = {
        metadata: data.metadata,
        headers: data.headers,
        data: data.data,
        response: message
      };

      response.metadata.time = new Date().toISOString();
      response.metadata.worker = response.metadata.owner;
      response.metadata.owner = this.config.TOPIC;
      response.metadata.mtype = 'output';

      var uowner = '';
      var uworker = '';
      if ('uowner' in response.metadata) {
        uowner = response.metadata.uowner;
      }

      if ('uworker' in response.metadata) {
        uworker = response.metadata.uworker;
      }

      response.metadata.uworker = uowner;
      response.metadata.uowner = uworker;

      var topic = 'respuesta_' + response.metadata.owner;

      if (response.metadata.bifurcacion) {
        topic = response.metadata.callback;
        response.metadata.bifurcacion = false;
      } else {
        response = {
          _id: response.metadata.id_transaction,
          response: response.response,
          metadata: response.metadata
        };
      }

      if (this.config.DEBUG) {
        console.log('SALIDA [' + topic + ']: ' + JSON.stringify(response));
      }

      response = Buffer.from(JSON.stringify(response)).toString('base64');

      this.producer.send([{
        topic: topic,
        messages: [response]
      }], function (err, _) {
        if (err) console.error(err);
      });
    }
  }, {
    key: 'message',
    value: function message(data) {
      var _this = this;

      var promise = new Promise(function (resolve, reject) {
        _this.listener({ resolve: resolve, reject: reject }, { authorization: data.headers.Authorizacion, data: data.data });
      });

      promise.then(function (response) {
        _this.send(data, {
          meta: {
            status: 'SUCCESS',
            id_transaction: data.metadata.id_transaction
          },
          data: {
            response: response
          }
        });
      }).catch(function (err) {
        if (!isNaN(err)) {
          var message = _this.getError(data.metadata.id_transaction, err);
          return _this.send(data, message);
        }

        console.error('ERR');
        console.error(err);
        _this.sendError(_this.config.TOPIC, data.metadata.id_transaction, err.message);
        _this.send(data, _this.getError(data.metadata.id_transaction, -99));
      });
    }
  }]);

  return Worker;
}();

exports.default = Worker;