'use strict'

class Fork {
  constructor (config) {
    this.config = config
  }

  setListener (listener) {
    this.listener = listener
  }

  setBifurcacion (bifurcacion) {
    this.bifurcacion = bifurcacion

    let exists = this.bifurcacion.find(obj => {
      let { appName, version, name } = obj

      if (!appName) {
        appName = this.config.APPLICATION_APP
      }

      if (!version) {
        version = this.config.APPLICATION_VERSION
      }

      let current = `${appName}_${version}_${name}`

      if (version === 'v1' && appName === 'appcoppel') {
        current = name
      }

      return current === this.config.TOPIC
    })

    if (!exists) {
      this.bifurcacion.unshift({
        appName: this.config.APPLICATION_APP,
        version: this.config.APPLICATION_VERSION,
        name: this.config.APPLICATION_NAME
      })
    }
  }

  getNextTopic (worker) {
    let { appName, version, name } = worker
    let current = `${appName}_${version}_${name}`

    if (version === 'v1' && this.config.APPLICATION_APP === 'appcoppel') {
      current = name
    }

    return { current, name }
  }

  send (data, response, final) {
    data.data = response

    let topic = data.metadata.worker
    if (final) {
      data = {
        _id: data.metadata.id_transaction,
        response: {
          data: {
            response: response
          },
          meta: {
            id_transaction: data.metadata.id_transaction,
            status: 'SUCCESS'
          }
        },
        metadata: data.metadata.metadata_inicial
      }
      topic = `respuesta_${data.metadata.callback}`

      if (response.meta) {
        data.response = response
      }
    }

    if (this.config.DEBUG) {
      console.log(`SALIDA [-> ${topic}]: ${JSON.stringify(data)}`)
    }
    response = Buffer.from(JSON.stringify(data)).toString('base64')

    this.producer.send([{
      topic,
      messages: [response]
    }], (err, _) => {
      if (err) console.error(err)
    })
  }

  error (data) {
    let response = {
      metadata: data.metadata,
      headers: data.headers,
      data: data.data,
      response: {
        errorCode: -99,
        userMessage: 'Ocurrio una excepcion, favor de intentar de nuevo.'
      }
    }

    response.metadata.time = (new Date()).toISOString()
    response.metadata.worker = response.metadata.owner
    response.metadata.owner = this.config.TOPIC
    response.metadata.mtype = 'output'

    let uowner = ''
    let uworker = ''
    if ('uowner' in response.metadata) {
      uowner = response.metadata.uowner
    }

    if ('uworker' in response.metadata) {
      uworker = response.metadata.uworker
    }

    response.metadata.uworker = uowner
    response.metadata.uowner = uworker

    let topic = `respuesta_${response.metadata.owner}`

    response = {
      _id: response.metadata.id_transaction,
      response: response.response,
      metadata: response.metadata
    }

    if (this.config.DEBUG) {
      console.log(`SALIDA [${this.config.TOPIC}]: ${JSON.stringify(response)}`)
    }

    response = Buffer.from(JSON.stringify(response)).toString('base64')

    this.producer.send([{
      topic,
      messages: [response]
    }], (err, _) => {
      if (err) console.error(err)
    })
  }

  message (data) {
    data.metadata.mtype = 'input'
    data.metadata.time = (new Date()).toISOString()
    data.metadata.bifurcacion = true

    const transactionId = data.metadata.id_transaction

    if (!('metadata_inicial' in data.metadata)) {
      data.metadata.metadata_inicial = Object.assign({}, data.metadata)
    }
    if (!('data_inicial' in data.metadata)) {
      data.metadata.data_inicial = Object.assign({}, data.data)
    }

    let size = this.bifurcacion.length
    let found = false

    for (let index = 0; index < size && !found; index++) {
      const obj = this.bifurcacion[index]
      const { current, name } = this.getNextTopic(obj)

      if (current === data.metadata.owner) {
        found = true

        if ((index + 1) < this.bifurcacion.length) {
          let row = this.getNextTopic(this.bifurcacion[index + 1])

          let operationId = new Date().getTime() / 1000000
          data.metadata.id_operacion = Math.floor(operationId)
          data.metadata.uowner = data.metadata.uworker
          data.metadata.worker = row.current
          data.metadata.uworker = `${data.metadata.worker}_${data.metadata.id_operacion}`
          data.metadata.owner = this.config.TOPIC
        }

        if (this.config.DEBUG) {
          if (index === 0) {
            console.log(`ENTRADA [${current}]: HEADERS: [${JSON.stringify(data.headers)}] DATA: [${JSON.stringify(data.data)}]`)
          } else {
            console.log(`ENTRADA [${current}]: HEADERS: [${JSON.stringify(data.headers)}] DATA: [${JSON.stringify(data.response)}]`)
          }
        }

        if (index > 0 && data.response && data.response.data && data.response.data.response && data.response.data.response.errorCode) {
          return this.send(data, data.response, true)
        }

        let promise = new Promise((resolve, reject) => {
          let initialData = data.metadata.data_inicial
          let response = data.data
          let microservice = name

          this.listener({ resolve, reject }, { microservice, transactionId, initialData, response })
        })

        promise.then(response => {
          if (response === null || response === undefined || JSON.stringify(response) === '{}') {
            response = data.data
          }
          this.send(data, response, ((index + 1) === size))
        })

        promise.catch(err => {
          console.error(err)
          this.sendError(this.config.TOPIC, err, transactionId)
          this.error({
            metadata: {
              owner: data.metadata.callback,
              id_transaction: transactionId
            },
            headers: {},
            data: {}
          })
        })
      }
    }
  }
}

export default Fork
