'use strict'

class Worker {
  constructor (config) {
    this.config = config
  }

  getError (transactionId, code) {
    let userMessage = ''
    if (!this.errors[code.toString()] || code.toString() === '-99') {
      userMessage = 'Ocurrio una excepcion, favor de intentar de nuevo.'
    } else {
      userMessage = this.errors[code.toString()]
    }

    let response = {
      meta: {
        status: 'ERROR',
        id_transaction: transactionId
      },
      data: {
        response: {
          errorCode: code,
          userMessage
        }
      }
    }
    return response
  }

  setErrors (errors) {
    this.errors = errors
  }

  setListener (listener) {
    this.listener = listener
  }

  send (data, message) {
    let response = {
      metadata: data.metadata,
      headers: data.headers,
      data: data.data,
      response: message
    }

    response.metadata.time = (new Date()).toISOString()
    response.metadata.worker = response.metadata.owner
    response.metadata.owner = this.config.TOPIC
    response.metadata.mtype = 'output'

    let uowner = ''
    let uworker = ''
    if ('uowner' in response.metadata) {
      uowner = response.metadata.uowner
    }

    if ('uworker' in response.metadata) {
      uworker = response.metadata.uworker
    }

    response.metadata.uworker = uowner
    response.metadata.uowner = uworker

    let topic = `respuesta_${response.metadata.owner}`

    if (response.metadata.bifurcacion) {
      topic = response.metadata.callback
      response.metadata.bifurcacion = false
    } else {
      response = {
        _id: response.metadata.id_transaction,
        response: response.response,
        metadata: response.metadata
      }
    }

    if (this.config.DEBUG) {
      console.log(`SALIDA [${topic}]: ${JSON.stringify(response)}`)
    }

    response = Buffer.from(JSON.stringify(response)).toString('base64')

    this.producer.send([{
      topic,
      messages: [response]
    }], (err, _) => {
      if (err) console.error(err)
    })
  }

  message (data) {
    let promise = new Promise((resolve, reject) => {
      this.listener({ resolve, reject }, { authorization: data.headers.Authorizacion, data: data.data })
    })

    promise.then(response => {
      this.send(data, {
        meta: {
          status: 'SUCCESS',
          id_transaction: data.metadata.id_transaction
        },
        data: {
          response
        }
      })
    }).catch(err => {
      if (!isNaN(err)) {
        let message = this.getError(data.metadata.id_transaction, err)
        return this.send(data, message)
      }

      console.error('ERR')
      console.error(err)
      this.sendError(this.config.TOPIC, data.metadata.id_transaction, err.message)
      this.send(data, this.getError(data.metadata.id_transaction, -99))
    })
  }
}

export default Worker
